﻿using PdfiumViewer;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Windows.Forms;

namespace LCDFontMaker
{
    public partial class MainForm : Form
    {

        private PdfDocument _lcd_document;// файл PDF
        private Image _OriginalImage;
        private Bitmap _TempImage;
        private Point _dot;
        private LcdFont _ocr_font = new LcdFont(); // шрифт, полученный из распознавания
        private LcdFont _file_font = new LcdFont();// шрифт, загруженный из файла
        private LanguageFontMaker lang = new LanguageFontMaker(); //языковые строковые данные
        private string dub = " ---";// здесь будет храниться адрес дубля знакоместа

        #region Init & finally
        public MainForm()
        {
            InitializeComponent();
            //формирование списка доступных языков интерейса
            List<string> ls = LanguageFunction.GetList();
            if (ls.Count > 0) //если языков несколько, то...
            {
                foreach (string l in ls)
                    itm_language.DropDownItems.Add(l);//... добавляем пункты в меню "Language"
                itm_language.DropDownItemClicked += new ToolStripItemClickedEventHandler(SetLanguage);
                SetLanguage(null, null);
            }
            else//если языков нет, скрываем меню
                itm_language.Visible = false;
        }
        /// <summary>
        /// Уничтожение ресурсов
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            _lcd_document?.Dispose();
            _OriginalImage?.Dispose();
            _TempImage?.Dispose();
        }
        #endregion Init & finally
        #region Languge
        /// <summary>
        /// Установка языка интерфейса
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SetLanguage(object sender, ToolStripItemClickedEventArgs e)  // установка языка интерфейса
        {
            string lng = e == null ? LanguageFunction.Default : e.ClickedItem.Text;
            if (lng != string.Empty)
            {
                lang.Default = false;
                LanguageFunction.Write(lang);
                lang = LanguageFunction.Read(lng);
                lang.Default = true;
                LanguageFunction.Write(lang);
            }
            foreach (ToolStripMenuItem i in itm_language.DropDownItems)
                i.Checked = (i.Text == lng);

            ZoomLcdChar.LanguageZoomLcdChar LangZEChar = new ZoomLcdChar.LanguageZoomLcdChar();
            // LangZEChar.Address = 
            // LangZEChar.Clear =
            // LangZEChar.Copy =
            // LangZEChar.Cut =
            // LangZEChar.Fill =
            // LangZEChar.FlipHoriz =
            // LangZEChar.FlipVert =
            // LangZEChar.Inverse =
            // LangZEChar.Paste =
            // LangZEChar.ShiftDown =
            // LangZEChar.ShiftLeft =
            // LangZEChar.ShiftRight =
            // LangZEChar.ShiftUp =
            zoomlcd.SetLanguage(LangZEChar);

            itm_file.Text = lang.File;
            itm_close.Text = lang.Exit;
            itm_language.Text = lang.Lang;
            itm_help.Text = lang.Help;
            itm_about.Text = lang.About;
            tbp_main.Text = lang.EditFontTable;
            tbp_image.Text = lang.ImgForOCR;
            gb_source.Text = lang.Source;
            rb_ocr.Text = lang.FromOCR;
            gb_prefsymbol.Text = lang.PrefSymbol;
            cb_user.Text = lang.UserSymbol;
            cb_dub.Text = lang.DublAddr + dub;
            lb_symbol.Text = lang.UserSymbol;
            gb_save.Text = lang.Save;
            lb_name.Text = lang.Name;
            bt_openpdf.Text = lang.OpenFile;
            bt_pageselect.Text = lang.SelectPage;
            gb_ocrpropery.Text = lang.PrefOCR;
            lb_mashtab.Text = lang.Scale;
            cb_proporc.Text = lang.Proporc;
            lb_sdvig.Text = lang.Offset;
            lb_doddot.Text = lang.DotDot;
            lb_symsym.Text = lang.SymbSymb;
            lb_colortrigger.Text = lang.TriggerCoor;
            cb_notused.Text = lang.SelUnusedSymb;
        }
        #endregion Language
        #region Menu
        /// <summary>
        /// Обработка событий главного меню
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void menu_ItemClicked(object sender, EventArgs e)
        {
            switch (((ToolStripMenuItem)sender).Name)
            {
                case "itm_close":
                    Application.Exit();
                    break;
                case "itm_about":
                    using (AboutBox aboutBox = new AboutBox()) aboutBox.ShowDialog();
                    break;
                case "itm_content":
                    break;
            }
        }
        #endregion Menu
        #region Load from OCR
        /// <summary>
        /// Вызов диалога для открытия файла PDF или изображения
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void bt_openpdf_Click(object sender, EventArgs e)
        {
            ofDlg.Filter = lang.DialogPdfImageFilter; // установка фильтра (на нужном языке) диалога открытия файла
            if (ofDlg.ShowDialog() != DialogResult.OK) return;
            tb_fileocr.Text = "";
            FileStream fs = new FileStream(ofDlg.FileName, FileMode.Open, FileAccess.Read);//читаем 4 байта для уверенности, что
            BinaryReader r = new BinaryReader(fs);                                         //пдф - это пдф. На самом деле, это
            uint signature = r.ReadUInt32();                                               //не 100% гарантия.
            fs.Close();
            r.Close();

            if (Path.GetExtension(ofDlg.FileName).ToLower() == ".pdf" || signature == 0x46445025) //если файл - это пдф...
            {
                try
                {
                    _lcd_document = PdfDocument.Load(this, ofDlg.FileName);// ... сохраняем его в _lcd_document
                    pdf_render.Load(_lcd_document);//...и загружаем его во вьювер и переключаемся на вкладку вьювера пдф
                    pdf_render.ZoomMode = PdfViewerZoomMode.FitWidth;// масштабирование по ширине
                    tabControl.SelectTab(tbp_pdf);
                }
                catch //(Exception ex)
                {
                    MessageBox.Show(lang.Message8, lang.Error);
                    return;
                }
            }
            else   //... если файл не PDF, то считаем файл изображением...
            {
                try
                {
                    _OriginalImage = Image.FromFile(ofDlg.FileName);
                }
                catch
                {
                    MessageBox.Show(lang.Message8, lang.Error);
                }
                if (_OriginalImage == null) return;
                ocr_picture_load();
                tabControl.SelectTab(tbp_image); //... и переключаемся на вкладку распознавания изображения
            }
            rb_ocr.Checked = true; // устанавливаем флажок выбора источника данных на "из распознавания"
            tb_fileocr.Text = ofDlg.FileName;
        }
        /// <summary>
        /// Если PDF файл отсутствует при переходе на вкладку, то отключаем элементы
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tabControl_Selected(object sender, TabControlEventArgs e)
        {
            if (e.TabPage == tbp_pdf) bt_pageselect.Visible = pdf_render.Document != null;
        }
        /// <summary>
        /// Выбор страницы PDF
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void bt_pageselect_Click(object sender, EventArgs e) //выбор страницы из пдф для распознавания
        {
            //Если _lcd_document не null, то получаем и сохраняем изображение страницы в _OriginalImage
            _OriginalImage = _lcd_document?.Render(pdf_render.Page, 2100, 2970, 100, 100, false);
            if (_OriginalImage == null) return;
            // если страница успешно сохранена в изображение, то вызываем его загрузку для обработки в OCR
            ocr_picture_load();
            tabControl.SelectTab(tbp_image);// переключаемся на страницу OCR
        }
        /// <summary>
        /// Загрузка картинки для распознавания и упраление вкл-выкл элементов управления
        /// </summary>
        private void ocr_picture_load()// загрузка картинки для распознавания
        {//отключим всё управление, ведь если картинки не будет -любое изменение несуществующего приведет к ошибке
            gb_ocrpropery.Enabled = false;
            ocr_picture.Image = _OriginalImage;
            nud_zoomx_ValueChanged(null, null); // вызываем масштабирование
            gb_ocrpropery.Enabled = true;//картинка загружена - включаем элементы управления
        }
        /// <summary>
        /// Установка пропорциональности масштабирования
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cb_proporc_CheckedChanged(object sender, EventArgs e)
        {
            nud_zoomy.Enabled = label3.Enabled = !cb_proporc.Checked; //делаем активными-неактивными элементы масштабирования по Y
        }
        /// <summary>
        /// Масштабирование картинки для распознавания
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void nud_zoomx_ValueChanged(object sender, EventArgs e)
        {
            //  if (_OriginalImage == null) return;// проверка безопасности ради. пока отключенана, так как проделана ранее вызова этой функции
            if (cb_proporc.Checked) nud_zoomy.Value = nud_zoomx.Value; // если пропорционально, то у = х
            // именяем размер PictureBox под новый масштабированный размер изображения, тем самым масштабируя и само изображение (свойствами PictureBox)
            ocr_picture.Width = (int)(_OriginalImage.Width / 100 * nud_zoomx.Value);
            ocr_picture.Height = (int)(_OriginalImage.Height / 100 * nud_zoomy.Value);
            //сохраняем отмасштабированный оригинал изображения в память для быстрого доступа к пикселам в _ocr_picture_Paint() (onPaint PictureBox)
            // так как доступ к пискселам изображения через свойство PictureBox значительно медленнее
            _TempImage = new Bitmap(_OriginalImage, (int)(_OriginalImage.Width / 100 * nud_zoomx.Value), (int)(_OriginalImage.Height / 100 * nud_zoomy.Value));
            // вызов перерисовки PictureBox
            ocr_picture.Invalidate();
        }
        /// <summary>
        /// onPaint PictureBox
        /// Отрисовка сетки "точек распознвания" на основе определения цвета пискела _TempImage
        /// Сохранение результата в _ocr_font.Chars[x, y].Dots[xx, yy] 
        /// Цвет дополнительно устанавливается в зависимости от типа знакоместа [обычные и пользователькие(неиспользуемые)]
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void _ocr_picture_Paint(object sender, PaintEventArgs e) //рисуем сетку точек и распознаем
        {
            if ((_dot.X > 1) && (_dot.Y > 1))  //здесь перебираются все точки всей таблицы символов
                for (int x = 0; x < 16; x++)   // с заданным шагом по горизонтали, вертикали и между знакоместами
                {
                    int dx = (int)Math.Round(x * (5 * (dotstepx.Value + 3) + dotcharx.Value / 6));
                    for (int y = 0; y < 16; y++)
                    {
                        int dy = (int)Math.Round(y * (8 * (dotstepy.Value + 3) + dotchary.Value / 6));
                        for (int xx = 0; xx < 5; xx++)
                            for (int yy = 0; yy < 8; yy++)
                            {
                                int _x = (int)(dx + _dot.X + dotdx.Value + xx * (dotstepx.Value + 3));
                                int _y = (int)(dy + _dot.Y + dotdy.Value + yy * (dotstepy.Value + 3));
                                if ((_x < _TempImage.Width - 4) && (_y < _TempImage.Height - 4) && (_x >= 0) && (_y >= 0))
                                {
                                    Color ccc = _TempImage.GetPixel(_x + 2, _y + 2);
                                    _ocr_font.Chars[x, y].Dots[xx, yy] = _ocr_font.Chars[x, y].User ? false : (ccc.R < nud_colortrigger.Value) && (ccc.G < nud_colortrigger.Value) && (ccc.B < nud_colortrigger.Value);
                                    e.Graphics.DrawRectangle(new Pen(_ocr_font.Chars[x, y].User ? Color.Blue : (_ocr_font.Chars[x, y].Dots[xx, yy] ? Color.YellowGreen : Color.Red)), _x, _y, 3, 3);
                                }
                            }
                    }
                }
        }
        /// <summary>
        /// Вкл-выкл режима "Указать неиспользуемые символы", сохрание выбранного режима через тип курсора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cb_notused_CheckedChanged(object sender, EventArgs e)
        {
            ocr_picture.Cursor = cb_notused.Checked ? Cursors.Hand : Cursors.Default;
            ocr_picture.Invalidate();
        }
        /// <summary>
        /// Выбор режима "Указать начальное положение сетки сканирования"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void bt_startgrid_Click(object sender, EventArgs e)
        {
            cb_notused.Checked = false; //отключаем режим "Указать неиспользуемые символы", затем меняем курсор. 
            ocr_picture.Cursor = Cursors.Cross; //В данном случае курсор является и флагом режима: каков курсор - таков режим.
            ocr_picture.Invalidate();
        }
        /// <summary>
        /// Обработка кликов мыши по картинке распознавания, а именно двух режимов:
        /// "Указать начальное положение сетки сканирования" и
        /// "Указать пользовательские символы" (они не будут распознаваться)
        /// Режим запоминается через курсор
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void _ocr_picture_MouseClick(object sender, MouseEventArgs e)
        {
            if (ocr_picture.Cursor == Cursors.Cross) //мы в режиме "Указать начальное положение сетки сканирования"
            {
                _dot.X = e.X; //запоминаем начальное положение
                _dot.Y = e.Y;
                dotdx.Value = dotdy.Value = 0; //сбрасывем смещение по икс-игрек - всё равно ведь заново придется подравнивать
                ocr_picture.Cursor = Cursors.Default; // выходим из режима, сбросом курсора (флага)
                ocr_picture.Invalidate();
                return;
            }
            if (ocr_picture.Cursor == Cursors.Hand)//мы в режиме "Указать пользовательские символы"
            { //всё ниже - это определение, по какому символу мы кликнули, или иначе - позиции курсора в клетках размером с символ
                var swidth = 5 * (dotstepx.Value + 3);
                var sheight = 8 * (dotstepy.Value + 3);
                int ax = -1;
                int ay = -1;
                for (int i = 0; i < 16; i++)
                {
                    int d = (int)Math.Round(i * (swidth + dotcharx.Value / 6));
                    if ((e.X - _dot.X >= d - 16) && (e.X - _dot.X <= d + swidth))
                    {
                        ax = i;
                        break;
                    }
                }
                for (int i = 0; i < 16; i++)
                {
                    int d = (int)Math.Round(i * (sheight + dotchary.Value / 6));
                    if ((e.Y - _dot.Y >= d - 16) && (e.Y - _dot.Y <= d + sheight))
                    {
                        ay = i;
                        break;
                    }
                }
                if ((ax < 0) || (ay < 0) || (ax > 15) || (ay > 15)) return;
                _ocr_font.Chars[ax, ay].User = !_ocr_font.Chars[ax, ay].User;
                ocr_picture.Invalidate();
            }
        }
        /// <summary>
        /// Обработка всех событий нумериковапдаунов по установке смещений-расстояний между точками/знакоместами и триггера цвета
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void dotY_SelectedIndexChanged(object sender, EventArgs e)
        {
            ocr_picture.Invalidate(); //просто вызываем перерисовку - всё анализируется в onPaint
        }
        /// <summary>
        /// Переключение типа всех знакомест нулевого столбца как пользовательские.
        /// Если до этого уже были указаны, то происходит инвертирование
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void bt_notusedcol_Click(object sender, EventArgs e)
        {
            for (int i = 0; i < 16; i++)
                _ocr_font.Chars[0, i].User = !_ocr_font.Chars[0, i].User;
            ocr_picture.Invalidate();
        }
        #endregion Load from OCR
        /// <summary>
        /// Переключение источника содержимого MapFont: распознавание или файл шрифта
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void rb_Select(object sender, EventArgs e)
        {   //сравнивается с сендером ради исключения двойного срабатывания события
            if (rb_ocr.Checked && (sender == rb_ocr))
                map_font.LcdFont = _ocr_font;
            else
                if (rb_file.Checked && (sender == rb_file))
                map_font.LcdFont = _file_font;
            else
                    if (sender == tabControl) //?непомню, зачем
            {
                if (rb_ocr.Checked)
                    map_font.LcdFont = _ocr_font;
                else
                    map_font.LcdFont = _file_font;
            }
            else
                return;
            zoomlcd.SetDots(map_font.SelChar.Dots);
            cb_user.Checked = map_font.SelChar.User;
            tb_symbol.Text = map_font.SelChar.Symbol.ToString();
            cb_dub.Checked = map_font.SelChar.DubAddr.X != -1;
        }
        #region Load from FileFont
        /// <summary>
        /// Вызов диалога выбора файла шрифта.
        /// В дальнейшем может потребоваться добавить проверку версий
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void bt_openfile_Click(object sender, EventArgs e)
        {
            ofDlg.Filter = lang.DialogFileFilter;
            if (ofDlg.ShowDialog() != DialogResult.OK) return;
            if (_file_font.ReadFromFile(ofDlg.FileName))
            {
                // если удалось прочитать файл, устанавливается на контролах его название, даташит если есть, 
                // переключается на режим "Из файла"
                lb_file.Text = ofDlg.FileName;
                lb_datasheet.Text = (_file_font.PdfDatasheet.Length > 0) ? lang.DatasheetEmbedded : "";
                tb_name.Text = _file_font.FontName;
                tb_comment.Text = _file_font.Comment;
                rb_file.Checked = true;
            }
            else
            {
                // сообщение об ошибке чтения файла шрифта
                MessageBox.Show(lang.Message8);
                _file_font = new LcdFont();
            }
        }
        #endregion Load from FileFont
        #region Work with MapFont
        /// <summary>
        /// Обработка изменений в окне редактирования символа
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void zoomlcd_ZoomLcdCharChanged(object sender, EventArgs e)
        {
            map_font.SelChar.Dots = zoomlcd.GetDots();
            // заполнение дублируемого символа по идее надо встроить в класс LcdFont
            if (map_font.SelChar.DubAddr.X != -1) //если есть дубликат, то меняем и ему значения, все, кроме адреса дубликата
                map_font.LcdFont.Chars[map_font.SelChar.DubAddr.X, map_font.SelChar.DubAddr.Y].Dots = zoomlcd.GetDots();
            map_font.Redraw();
            map_to_font();
        }
        /// <summary>
        /// Обработка выделения знакоместа на карте
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void map_font_SelectedChar(object sender, MapLcdFont.SelectedCharEventArgs e)
        {
            // направляем данные в окно редактирования символа
            zoomlcd.Address = e.Address;
            cb_user.Checked = e.LcdChar.User;
            // определение режима (курсор - флаг)
            if (Cursor == Cursors.Cross)//если "Установка дублирования знакоместа "символа"
            {  // то проверяем, не то же саме ли это знакоместо, и если нет - устанавливаем адрес и копируем свойства
                Point a = (Point)map_font.Tag;
                if (a == e.Address) return;
                Cursor = map_font.Cursor = Cursors.Default;// Сброс режима "Установка дублирования знакоместа "символа"
                e.LcdChar.DubAddr = a;
                map_font.LcdFont.Chars[a.X, a.Y].DubAddr = e.Address;
                // копирование знакоместа дубля
                e.LcdChar.Dots = map_font.LcdFont.Chars[a.X, a.Y].Dots;
                e.LcdChar.Symbol = map_font.LcdFont.Chars[a.X, a.Y].Symbol;
                e.LcdChar.User = map_font.LcdFont.Chars[a.X, a.Y].User;
            }
            else
            {
                bool b = e.LcdChar.DubAddr.X != -1;
                if (b != cb_dub.Checked) cb_dub.Checked = b;
                zoomlcd.SetDots(e.LcdChar.Dots);
            }
            tb_symbol.Text = e.LcdChar.Symbol.ToString();
            dub = e.LcdChar.DubAddr.X == -1 ? " ---" : " 0x" + e.LcdChar.DubAddr.X.ToString("X") + e.LcdChar.DubAddr.Y.ToString("X");
            cb_dub.Text = lang.DublAddr + dub;
        }
        /// <summary>
        /// сохранение данных с MapFont в текущий шрифт 
        /// </summary>
        private void map_to_font()
        {
            if (rb_ocr.Checked)
                _ocr_font = map_font.LcdFont;
            else
                _file_font = map_font.LcdFont;
        }
        private void cb_user_CheckedChanged(object sender, EventArgs e)
        {
            map_font.SelChar.User = cb_user.Checked;
            Point a = map_font.SelChar.DubAddr;
            if ((a.X > -1) && (a.Y > -1)) map_font.LcdFont.Chars[a.X, a.Y].User = cb_user.Checked;
            map_font.Redraw();
            map_to_font();
        }
        /// <summary>
        /// установка флага "Событие вызвано мышью"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cb_dub_MouseDown(object sender, MouseEventArgs e)
        {
            cb_dub.Tag = 1;
        }
        /// <summary>
        /// Обработка чекбокса, устанавливающего режим "Установка дублирования знакоместа "символа" 
        /// или сбрасывающего признак налия дубля у знакоместа
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cb_dub_CheckedChanged(object sender, EventArgs e)
        {//определение, кто вызвал событие, мышь или программа, избегая возможной рекурсии вызовов
            if (cb_dub.Tag == null) return; //если программа - выходим
            cb_dub.Tag = null;
            if (cb_dub.Checked)
            { // если это установка режима, то ставим флаг режима (курсор) и сохраняем адрес текущего знакоместа
                Cursor = map_font.Cursor = Cursors.Cross;
                map_font.Tag = new Point(map_font.PosX, map_font.PosY);
            }
            else
            {
                Cursor = map_font.Cursor = Cursors.Default;
                map_font.SelChar.DubAddr = new Point(-1);
                cb_dub.Text = lang.DublAddr + " ---";
                Point a = new Point(map_font.PosX, map_font.PosY);
                for (int x = 0; x < 16; x++)
                    for (int y = 0; y < 16; y++)
                        if (a == map_font.LcdFont.Chars[x, y].DubAddr)
                            map_font.LcdFont.Chars[x, y].DubAddr = new Point(-1);
                map_font.Redraw();
                map_to_font();
            }
        }
        #endregion Work with MapFont
        #region Property and Save

        /// <summary>
        /// Управление даташитом шрифта (сохранение, удаление и выгрузка файла PDF )
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void bt_datasheet_Click(object sender, EventArgs e)
        {
            if (((Button)sender).Name == "bt_datasheet_clear")
            {
                map_font.LcdFont.PdfDatasheet = new MemoryStream();
                lb_datasheet.Text = "";
            }
            else if (((Button)sender).Name == "bt_datasheet_load")
            {
                ofDlg.Filter = lang.DialogPdfImageFilter;
                if (ofDlg.ShowDialog() != DialogResult.OK) return;
                try
                {
                    FileStream fstream = new FileStream(ofDlg.FileName, FileMode.Open, FileAccess.Read);
                    byte[] b = new byte[fstream.Length];
                    fstream.Read(b, 0, (int)fstream.Length);
                    map_font.LcdFont.PdfDatasheet?.Close();
                    map_font.LcdFont.PdfDatasheet = new MemoryStream(b);
                    fstream.Close();
                    lb_datasheet.Text = ofDlg.FileName;
                }
                catch { }
            }
            else
            {
                sfDlg.Filter = lang.DialogPdfFilter;
                sfDlg.DefaultExt = "pdf";
                if (sfDlg.ShowDialog() != DialogResult.OK) return;
                try
                {
                    FileStream file = new FileStream(sfDlg.FileName, FileMode.Create, FileAccess.Write);
                    byte[] b = map_font.LcdFont.PdfDatasheet.ToArray();
                    file.Write(b, 0, b.Length);
                    file.Close();
                }
                catch { }
            }
            map_to_font();
        }
        /// <summary>
        /// Управление видимостью кнопок "удалить" и "выгрузить" даташит
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void lb_datasheet_TextChanged(object sender, EventArgs e)
        {
            bt_datasheet_clear.Visible = bt_datasheet_unload.Visible = (lb_datasheet.Text != "");
        }
        /// <summary>
        /// Сохранение наименования шрифта
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tb_name_TextChanged(object sender, EventArgs e)
        {
            map_font.LcdFont.FontName = tb_name.Text;
            map_to_font();
        }
        /// <summary>
        /// Сохранение комментария шрифта
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tb_comment_TextChanged(object sender, EventArgs e)
        {
            map_font.LcdFont.Comment = tb_comment.Text;
            map_to_font();
        }
        /// <summary>
        /// Сохранение данных шрифта в файл
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void bt_savetofile_Click(object sender, EventArgs e)
        {
            sfDlg.Filter = lang.DialogFileFilter;
            sfDlg.DefaultExt = "lcd";
            if (sfDlg.ShowDialog() == DialogResult.OK) map_font.LcdFont.SaveToFile(sfDlg.FileName);
        }
        /// <summary>
        /// Сопоставление знакоместу символа ОС 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tb_symbol_TextChanged(object sender, EventArgs e)
        {
            map_font.SelChar.Symbol = tb_symbol.Text.Length > 0 ? tb_symbol.Text[0] : '\0';
            map_to_font();
        }
        #endregion Property and Save
    }
}
